import numpy as np 
import pandas as pd

#------------ Random Object Class----------
class RandomObject(object):
    '''Class of object that simulates a wide variety of random objects'''

    def __init__(self,probs=None,results=None):
        '''Initiates the RandomObject instance
        Arguments:
            probs: list of probabilities associated to the results
            results: list of the names of the results of the random object
        '''
        assert len(probs)==len(results), 'There is a missmatch between the probs and results'
        assert np.array(probs).sum()==1, "The probabilities don't add up to 1"
        self.probabilities = np.array(probs)
        self.results = np.array(results)
    
    def get_probablities(self):
        '''Get the probabilities of the RandomObject'''
        return self.probabilities

    def set_probabilities(self,new_probabilities):
        ''' Sets the probabilities of the object'''
        assert len(new_probabilities)==self.get_results() and np.array(new_probabilities).sum()==1,'The new probabilities are not valid'
        self.probabilities = np.array(new_probabilities)
    
    def get_results(self):
        '''Get the results of the RandomObject instance'''
        return self.results
    
    def set_results(self,new_results):
        '''Set new list of results to the RandomObject'''
        assert len(new_results)==len(self.get_results()),'Results introduced are not valid'
        self.results = new_results

    def get_generator(self,throws=2):
        '''Generates a RandomObject generator
        Arguments:
            throws: the number of throws or iterations of the random object
        
        Output: generator object that yields all possible combinations for the 
                RandomObject, giving a tuple with dictionary with the number of 
                appearances of each result given the number of throws, and the 
                probability of that event
        '''
        results = self.get_results() #store the results of the RandomObject
        num_results = len(results)
        for i in range(num_results**throws):
           
            #Create a dictionary with a count of the results
            generated = {result:0 for result in results}
            for j in range(throws):
                for k in range(num_results):
                    if i//(num_results**j)%num_results == k:
                        generated[results[k]] += 1
           
           #Calculate the probability of that specific throw
            probability_throw = (self.get_probablities()**np.array(list(generated.values()))).prod()
           
            yield (generated,probability_throw)

    def see_table(self,throws=2):
        '''Produces a table with all the possible results given the number of
        throws

        Arguments:
            throws: number of throws simulated
        '''

        # Create a data frame to store the results of each throw
        column_names = self.get_results().tolist()
        index_names = ['Combination '+str(throw+1) for throw in range(throws)]+['Total']
        df = pd.DataFrame(columns=column_names,index=index_names)
        
        # Create a generator of the RandomObject
        random_object_generator = self.get_generator(throws)

        # Fill the table with the results of all possible outcomes
        throw = 0 
        
        while True: 
            #Exhaust all possible options
            try:
                throw += 1
                (generated,probability) = next(random_object_generator)
                df.loc['Combination '+str(throw),'Probability'] = probability
                for result in generated.keys():
                    df.loc['Combination '+str(throw),result] = generated[result]
            #Get totals
            except:
                df.loc['Total','Probability'] = df['Probability'].sum()
                for result in generated.keys():
                    df.loc['Total',result] = df[result].sum()
                return df.sort_index()
#----------------------------------------------------------------------------------
#----------------------------------------------------------------------------------
def calculate_bayes(prob_a,sensitivity,specificity,b=True):
    '''Calculates the Bayes algorithm for an event A and a test B
    prob_a: float between 0 and 1 that represents the probability of event A
    sensitivity: probability of B given that A occurred
    specificity: probability of not B given that not A occured
    '''
    
    # Calculating the complements of the information we know
    prob_nota = 1-prob_a
    prob_notb_a = 1-sensitivity # probability of not B given A
    prob_b_nota = 1-specificity # probability of B given not A
    
    # If the test is B:
    if b:
        prob_a_join_b = prob_a*sensitivity
        prob_nota_join_b = prob_nota*prob_b_nota
        prob_b = prob_a_join_b+prob_nota_join_b
        prob_a_b = prob_a_join_b/prob_b       #probability of A given B
        prob_nota_b = prob_nota_join_b/prob_b #probability of not A given B
        return (prob_a_b,prob_nota_b)
    
    # If the test is not B:
    else:
        prob_a_join_notb = prob_a*prob_notb_a
        prob_nota_join_notb = prob_nota*specificity
        prob_notb = prob_a_join_notb+prob_nota_join_notb
        prob_a_notb = prob_a_join_notb/prob_notb       # probability of A given not B
        prob_nota_notb = prob_nota_join_notb/prob_notb #probability of not A given not B
        return (prob_a_notb,prob_nota_notb)







#-------------------------------- Gaussian Class -------------------------------
import math
import matplotlib.pyplot as plt
from .Generaldistribution import Distribution
import numpy as np

class Gaussian(Distribution):
    """ Gaussian distribution class for calculating and 
    visualizing a Gaussian distribution.
    
    Attributes:
        mean (float) representing the mean value of the distribution
        stdev (float) representing the standard deviation of the distribution
        data_list (list of floats) a list of floats extracted from the data file
            
    """
    def __init__(self, mu = 0, sigma = 1):
        
        Distribution.__init__(self,mu,sigma)


    
    def calculate_mean(self):
    
        """Method to calculate the mean of the data set.
        
        Args: 
            None
        
        Returns: 
            float: mean of the data set
    
        """
        mean = float(sum(self.data)/len(self.data))
        self.mean = mean
        return self.mean


    def calculate_stdev(self, sample=True):

        """Method to calculate the standard deviation of the data set.
        
        Args: 
            sample (bool): whether the data represents a sample or population
        
        Returns: 
            float: standard deviation of the data set
    
        """
        numerator = np.power((np.array(self.data)-self.mean),2).sum()
        if sample:
            std = np.sqrt(numerator/(len(self.data)-1))
        else:
            std = np.sqrt(numerator/len(self.data))
        self.stdev = std
        return self.stdev
        


    def plot_histogram(self):
        """Method to output a histogram of the instance variable data using 
        matplotlib pyplot library.
        
        Args:
            None
            
        Returns:
            None
        """
        plt.figure(figsize=(10,5))
        plt.hist(self.data,bins=30,color='teal')
        plt.xlabel('Data')
        plt.ylabel('Frequency')
        plt.title('Distribution of the Data')
        plt.show()
        # TODO: Plot a histogram of the data_list using the matplotlib package.
        #       Be sure to label the x and y axes and also give the chart a title
        
                
        
    def pdf(self, x):
        """Probability density function calculator for the gaussian distribution.
        
        Args:
            x (float): point for calculating the probability density function
            
        
        Returns:
            float: probability density function output
        """
        return (1/(self.stdev*np.sqrt(2*np.pi)))*np.exp(-0.5*((x-self.mean)\
                                                             /self.stdev)**2)
        # TODO: Calculate the probability density function of the Gaussian distribution
        #       at the value x. You'll need to use self.stdev and self.mean to do the calculation       

    def plot_histogram_pdf(self, n_spaces = 50):

        """Method to plot the normalized histogram of the data and a plot of the 
        probability density function along the same range
        
        Args:
            n_spaces (int): number of data points 
        
        Returns:
            list: x values for the pdf plot
            list: y values for the pdf plot
            
        """
        
        #TODO: Nothing to do for this method. Try it out and see how it works.

        min_range = min(self.data)
        max_range = max(self.data)
        
         # calculates the interval between x values
        interval = 1.0 * (max_range - min_range) / n_spaces

        x = []
        y = []
        
        # calculate the x values to visualize
        for i in range(n_spaces):
            tmp = min_range + interval*i
            x.append(tmp)
            y.append(self.pdf(tmp))

        # make the plots
        fig, axes = plt.subplots(2,sharex=True)
        fig.subplots_adjust(hspace=.5)
        axes[0].hist(self.data, density=True)
        axes[0].set_title('Normed Histogram of Data')
        axes[0].set_ylabel('Density')

        axes[1].plot(x, y)
        axes[1].set_title('Normal Distribution for \n Sample Mean and Sample Standard Deviation')
        axes[0].set_ylabel('Density')
        plt.show()

        return x, y
    
    def __add__(self,other):
        '''Function to add together two Gaussian distributions
        
        Arguments:
            other (Gaussian): a Gaussian instance
        
        Output:
            Gaussian: Gaussian distribution
        '''
        result = Gaussian()
        result.mean = self.mean + other.mean
        result.stdev = np.sqrt(self.stdev**2 + other.stdev**2)
        
        return result
    
    def __repr__(self):
        '''Function to output the characteristics of the Gaussian instance
        
        Arguments:
            None
        Output:
            string: characteristic of the Gaussian
        '''
        return 'Mean {}, Standard Deviation {}'.format(self.mean,self.stdev)




from setuptools import setup
import setuptools
with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
setup(name='probadg',
      version = '0.0.1',
      author = 'Alejandro Duenas Garcia',
      author_email = 'alejandro19971231@gmail.com',
      long_description = long_description,
      long_description_content_type = 'text/markdown',
      description = 'Binomial and Gaussian objects',
      url = "https://github.com/Alejandro-Duenas/probadg",
      packages = setuptools.find_packages(),
      classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
      python_requires='>=3.6',
      zip_safe = False)